import multiprocessing as mp
import os
import random

from config import DATASET_PATH, BUNDLE_DATASET_PATH, TEST_SET_SPLIT, LINES_PER_IMAGE, N_SAMPLES
from utils import get_source_code_from_file


def create_bundles(lang):
    files_list = os.listdir(os.path.join(DATASET_PATH, lang))
    n_files = len(files_list)

    # if there are more than 1000 file select randomly 1000 files
    if n_files > 1000:
        random.seed(123)
        files_list = random.sample(files_list, 1000)

    with open(os.path.join(BUNDLE_DATASET_PATH, lang + '.txt'), 'a') as f:
        for sha in files_list:
            content = get_source_code_from_file(os.path.join(DATASET_PATH, lang, sha))
            f.write(content)
            f.write('\n')


print('Creating bundles...')

languages = os.listdir(DATASET_PATH)

if not os.path.exists(BUNDLE_DATASET_PATH):
    os.mkdir(BUNDLE_DATASET_PATH)

# create bundle files
for lan in languages:
    open(os.path.join(BUNDLE_DATASET_PATH, lan + '.txt'), 'w').close()

pool = mp.Pool(mp.cpu_count())
pool.map(create_bundles, languages)
pool.close()

############################

print('Excluding too short bundles...')

# To be selected a language must have a number of lines of code greater the double of the total
# number of lines needed to build the source codes images for the test set
selected_languages = []
for src_file in os.listdir(BUNDLE_DATASET_PATH):
    with open(os.path.join(BUNDLE_DATASET_PATH, src_file), 'r') as f:
        src_file_length = len(f.readlines())
    if src_file_length < N_SAMPLES * TEST_SET_SPLIT * LINES_PER_IMAGE * 2:
        os.remove(os.path.join(BUNDLE_DATASET_PATH, src_file))
