import multiprocessing as mp
import os
import pickle
from shutil import copyfile

import pandas
import yaml
from tqdm import tqdm

from config import DATASET_PATH, TEMP_FOLDER, GITHUB_DATASET_PATH
from utils import linguist, get_source_code


def distinguish_par(sha):
    # decrypt SHA1 file
    content = get_source_code(sha)

    # save the content of the file to recognize it with Linguist
    source_code_path = os.path.join(TEMP_FOLDER, sha + '.' + ext)
    with open(source_code_path, 'w') as file:
        file.write(content)

    ret_lang = linguist(source_code_path)
    os.remove(source_code_path)

    if ret_lang == 'No language found!':
        print('No language found!')
        # pass
    else:
        output_file_path = os.path.join(DATASET_PATH, ret_lang, sha)
        if not os.path.exists(os.path.join(DATASET_PATH, ret_lang)):
            os.mkdir(os.path.join(DATASET_PATH, ret_lang))
        src_tmp = os.path.join(GITHUB_DATASET_PATH, sha[:2], sha[2:4], sha)
        copyfile(src_tmp, output_file_path)


print("Creating ground truth...")

data = pandas.read_csv('files_list/single_labels.csv', names=['sha', 'label'], encoding='utf-8')

with open('results/non_rare_extensions.pkl', 'rb') as f:  # save the frequency values
    selected_extensions = pickle.load(f)

if not os.path.exists(DATASET_PATH):
    os.mkdir(DATASET_PATH)
if not os.path.exists(TEMP_FOLDER):
    os.mkdir(TEMP_FOLDER)

with open("github_linguist_languages_file.yml", 'r') as yaml_file:
    linguist_languages = yaml.load(yaml_file, Loader=yaml.FullLoader)

for lang in linguist_languages:
    tmp_path = os.path.join(DATASET_PATH, lang)
    if not os.path.exists(tmp_path):
        os.mkdir(tmp_path)

for ext in tqdm(selected_extensions):
    data_ext = data[data['label'] == ext]
    extension_files = data_ext['sha'].values

    # to avoid starting from zero in case of interruption
    done_sha_list = []
    for fold in os.listdir(DATASET_PATH):
        for done_sha in os.listdir(os.path.join(DATASET_PATH, fold)):
            done_sha_list.append(done_sha)

    # exclude already processed files
    extension_files = set(extension_files) - set(done_sha_list)

    pool = mp.Pool(mp.cpu_count())
    pool.map(distinguish_par, extension_files)
    pool.close()

    os.remove(TEMP_FOLDER)
