import multiprocessing as mp
import os

from PIL import ImageFont

from config import *
from utils import scramble_all_x, scramble_alphanumsym, scramble_alphanum, scramble_alphasym, \
    scramble_numsym, scramble_alpha, scramble_num, scramble_sym, render_text


def render_par(i):
    train_set_size = N_SAMPLES - test_set_size
    test_set_lines = test_set_size * LINES_PER_IMAGE

    start = len(content) - test_set_lines + LINES_PER_IMAGE * (i - train_set_size)
    text = ''.join(content[start: start + LINES_PER_IMAGE])

    text_scrambled = ''
    j = 0
    while j < len(text):
        letter = text[j]
        if all_x:
            text_scrambled += scramble_all_x(letter)
        elif alpha_scramble and num_scramble and sym_scramble:
            text_scrambled += scramble_alphanumsym(letter)
        elif alpha_scramble and num_scramble:
            text_scrambled += scramble_alphanum(letter)
        elif alpha_scramble and sym_scramble:
            text_scrambled += scramble_alphasym(letter)
        elif num_scramble and sym_scramble:
            text_scrambled += scramble_numsym(letter)
        elif alpha_scramble:
            text_scrambled += scramble_alpha(letter)
        elif num_scramble:
            text_scrambled += scramble_num(letter)
        elif sym_scramble:
            text_scrambled += scramble_sym(letter)
        else:
            text_scrambled += letter
        j += 1

    image_path = os.path.join(OUTPUT_DIR_PATH, lang, ("%04d.png" % i))
    render_text(text_scrambled, image_path, font)


#######################################################################

font = ImageFont.truetype(FONT_NAME, FONT_SIZE)

scramble_type_list = ['alpha', 'num', 'sym', 'alphanum', 'alphasym', 'numsym', 'alphanumsym', 'all_x']

languages = []
for lang_tmp in os.listdir(BUNDLE_DATASET_PATH):
    languages.append(lang_tmp[:-4])

print("Number of languages:", str(len(languages)))

index_lang = 1
for lang in languages:
    print(str(index_lang) + '. ' + lang)

    with open(os.path.join(BUNDLE_DATASET_PATH, lang + '.txt')) as file:
        content = file.readlines()

    for scramble_type in scramble_type_list:
        print('   ' + scramble_type)
        OUTPUT_DIR_PATH = RENDERINGS_OUTPUT_FOLDER + '_' + scramble_type.upper()

        if not os.path.exists(OUTPUT_DIR_PATH):
            os.mkdir(OUTPUT_DIR_PATH)

        num_scramble = False
        alpha_scramble = False
        sym_scramble = False
        all_x = False

        if 'num' in scramble_type:
            num_scramble = True
        if 'sym' in scramble_type:
            sym_scramble = True
        if 'alpha' in scramble_type:
            alpha_scramble = True
        if 'all_x' in scramble_type:
            all_x = True

        out_path_lang = os.path.join(OUTPUT_DIR_PATH, lang)
        if not os.path.exists(out_path_lang):
            os.mkdir(out_path_lang)

        test_set_size = int(N_SAMPLES * TEST_SET_SPLIT)
        start_test_set = N_SAMPLES - test_set_size
        test_set_files_index = range(start_test_set, N_SAMPLES)

        pool = mp.Pool(mp.cpu_count())
        pool.map(render_par, test_set_files_index)
        pool.close()

    index_lang += 1
